#include "OrderProcess.h"


#include "Inventory.h"
#include "CustomerRegister.h"
#include <stdio.h>
#include <stdio.h>

static float OrderProcess_processOrder(struct Inventory_Item items[],struct CustomerRegister_Customer customer);

static int8_t OrderProcess_checkInventoryAndValidateCustomer(struct Inventory_Item items[],struct CustomerRegister_Customer customer);

static float OrderProcess_calculatePriceAndApplyDiscount(struct Inventory_Item items[]);

static float OrderProcess_addTwentyFivePercentDiscount(float total);

static float OrderProcess_addFiftyPercentDiscount(float total);

static float OrderProcess_addSeventyFivePercentDiscount(float total);

static float OrderProcess_processOrder(struct Inventory_Item items[], struct CustomerRegister_Customer customer) 
{
  /* Step 1 */
  OrderProcess_checkInventoryAndValidateCustomer(items, customer);
  
  /* Step 2 */
  float total = OrderProcess_calculatePriceAndApplyDiscount(items);
  
  /* Step 3 */
  if ( total < ORDERPROCESS_FREE_SHIPPING_LIMIT ) 
  {
    float shippingFee = 75;
    total = total + shippingFee;
  }

  return total;
}


static int8_t OrderProcess_checkInventoryAndValidateCustomer(struct Inventory_Item items[], struct CustomerRegister_Customer customer) 
{
  int8_t isItemInStock = 0;
  int8_t counter = 0;
  for ( int8_t i = 0; i < ORDERPROCESS_ORDER_SIZE; i++ )
  {
    for ( int8_t j = 1; j < INVENTORY_INVENTORY_SIZE; j++ )
    {
      if ( Inventory_items[i].itemNo == Inventory_items[j].itemNo ) 
      {
        counter++;
      }

    }

  }

  (counter == ORDERPROCESS_ORDER_SIZE) ? (isItemInStock = 1) : (isItemInStock = 0);
  
  int8_t isCustomerValid = 0;
  for ( int8_t i = 0; i < CUSTOMERREGISTER_CUSTOMER_REGISTRY_SIZE; i++ )
  {
    if ( CustomerRegister_customers[i].name == customer.name ) 
    {
      if ( CustomerRegister_customers[i].status == CustomerRegister_Status__GOOD ) 
      {
        isCustomerValid = 1;
      }

    }

  }

  return isItemInStock && isCustomerValid;
}


static float OrderProcess_calculatePriceAndApplyDiscount(struct Inventory_Item items[]) 
{
  float runningTotal = 0.0;
  for ( int8_t i = 0; i < ORDERPROCESS_ORDER_SIZE; i++ )
  {
    if ( items[i].type == Inventory_Type__AMETHYST ) 
    {
      runningTotal += items[i].carat * 12000;
    }
    else 
    {
      if ( items[i].type == Inventory_Type__DIAMOND ) 
      {
        runningTotal += items[i].carat * 16000;
      }
      else 
      {
        if ( items[i].type == Inventory_Type__EMERALD ) 
        {
          runningTotal += items[i].carat * 13000;
        }
        else 
        {
          if ( items[i].type == Inventory_Type__RUBY ) 
          {
            runningTotal += items[i].carat * 13200;
          }
          else 
          {
            if ( items[i].type == Inventory_Type__SAPPHIRE ) 
            {
              runningTotal += items[i].carat * 10100;
            }

          }

        }

      }

    }

  }

  
  if ( runningTotal > 20000 ) 
  {
    runningTotal = OrderProcess_addSeventyFivePercentDiscount(runningTotal);
  }
 else if (runningTotal > 10000) {
    runningTotal = OrderProcess_addFiftyPercentDiscount(runningTotal);
  }
 else if (runningTotal > 5000) {
    runningTotal = OrderProcess_addTwentyFivePercentDiscount(runningTotal);
  }

  return runningTotal;
}


static float OrderProcess_addTwentyFivePercentDiscount(float total) 
{
  return total * ORDERPROCESS_MINOR_DISCOUNT;
}


static float OrderProcess_addFiftyPercentDiscount(float total) 
{
  return total * ORDERPROCESS_MEDIUM_DISCOUNT;
}


static float OrderProcess_addSeventyFivePercentDiscount(float total) 
{
  return total * ORDERPROCESS_MAJOR_DISCOUNT;
}


/* - - */
int8_t OrderProcess_testProcessOrder(void) 
{
  int8_t __failuresVal = 0;
  int8_t* __failures = &__failuresVal;
  printf("$$runningTest: running test (");
  printf(") @OrderProcess:testProcessOrder#6986755912720911055\n");
  
  struct Inventory_Item orderOneItems[ORDERPROCESS_ORDER_SIZE] = {Inventory_items[0],Inventory_items[1],Inventory_items[2]};
  struct Inventory_Item orderTwoItems[ORDERPROCESS_ORDER_SIZE] = {Inventory_items[1],Inventory_items[2],Inventory_items[3]};
  struct Inventory_Item orderThreeItems[ORDERPROCESS_ORDER_SIZE] = {Inventory_items[2],Inventory_items[3],Inventory_items[4]};
  struct Inventory_Item orderFourItems[ORDERPROCESS_ORDER_SIZE] = {Inventory_items[3],Inventory_items[4],Inventory_items[0]};
  
  if ( !(OrderProcess_processOrder(orderOneItems, CustomerRegister_customers[0]) == 9375.000000) ) 
  {
    (*__failures)++;;
    printf("$$FAILED: [1;31m***FAILED***[0m (");
    printf("testID=%i",0);
    printf(") @OrderProcess:testProcessOrder#6986755912720918878\n");
    
  }

  if ( !(OrderProcess_processOrder(orderTwoItems, CustomerRegister_customers[1]) == 13050.000000) ) 
  {
    (*__failures)++;;
    printf("$$FAILED: [1;31m***FAILED***[0m (");
    printf("testID=%i",1);
    printf(") @OrderProcess:testProcessOrder:1#5760004206824075541\n");
    
  }

  if ( !(OrderProcess_processOrder(orderThreeItems, CustomerRegister_customers[2]) == -1) ) 
  {
    (*__failures)++;;
    printf("$$FAILED: [1;31m***FAILED***[0m (");
    printf("testID=%i",2);
    printf(") @OrderProcess:testProcessOrder:2#5760004206824076000\n");
    
  }

  if ( !(OrderProcess_processOrder(orderFourItems, CustomerRegister_customers[3]) == 13068.750000) ) 
  {
    (*__failures)++;;
    printf("$$FAILED: [1;31m***FAILED***[0m (");
    printf("testID=%i",3);
    printf(") @OrderProcess:testProcessOrder:3#5760004206824076472\n");
    
  }

  return __failuresVal;
}


